#!/usr/bin/perl -w

#Copyright (c) Nmap project and its contributor
#All rights reserved.

#Redistribution and use in source and binary forms, with or without 
#modification, are permitted provided that the following conditions 
#are met:

# 1. Redistributions of source code must retain the above copyright 
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in 
#    the documentation and/or other materials provided with the 
#    distribution.
# 3. Neither the name of Nmap nor the names of its contributors 
#    may be used to endorse or promote products derived from this 
#    software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.


package Nmap::CGI::Main;
require Exporter;
@ISA 	= qw(Exporter);
@EXPORT = qw( output  open_template init_web check_param);

use strict;
use CGI;
use Email::Valid;
use HTML::Template;
use Nmap::Main;
use Nmap::CGI::Auth;
use Nmap::User;
use Nmap::Scan;
use Nmap::Logger;

use constant
{
	TEMPLATE_DIR => "/usr/share/nmap-cgi/templates/",

};

my $main = init Nmap::Main;

sub init_web
{
	my $cgi;
	my $user;
	my $opts;
	my $query;
	my $stmt;
	my $adr;
	my $tmp;

	$cgi = new CGI;
	$opts = shift;

	if( ( ! defined( $main->{'init'} ) ) || ( $main->{'init'} == 0 ) )
	{
		output( open_template( 'close' )->output  , { WARN => "Database connection error" } );
	}


	$user = authuser( $main );

	if( $main->get_config->get_open == 0 )
	{
		output( open_template( 'close' )->output  , undef );
	}

	if( defined $opts )
	{
		if( ( $opts->{'MUST-AUTH'} == 1 ) && ( ! defined( $user ) ) )
		{
			output( '' , { WARN => "You must be logged" }  );
		}

		if( $opts->{'MUST-ADMIN'} == 1 )
		{
			if( ! defined( $user ) )
			{
				output( undef , { WARN => "You must be logged as admin" } );
			}
			elsif( $user->get( 'state' ) != Nmap::User::STATE_ADMIN )
			{
				output( open_template( 'homepage' )->output , { WARN => "You must be admin" } );
			}
		}

		#Log the IP adress if the user is auth
		if( ( $opts->{'MUST-AUTH'} == 1 ) || ( $opts->{'MUST-ADMIN'} == 1 ) )
		{
			$adr = $ENV{'REMOTE_ADDR'};
			if( defined $adr )
			{
				$query = "SELECT adr FROM userips WHERE userid=? AND adr = ?";
				$stmt = $main->get_db->prepare( $query );
				$tmp = $stmt->execute( $user->get('id') , $ENV{'REMOTE_ADDR'} );

				if( $tmp == 0 )
				{	
					$query = "INSERT INTO userips( userid , adr ) VALUES( ? , ?) ";
					$stmt = $main->get_db->prepare( $query );
					$stmt->execute( $user->get('id') , $adr ) or nmaplog( "Cannot log IP" );
				}
			}
		}

	}

	return( ( $main , $cgi , $user ) );
}

sub open_template
{
	my $name;
	my $subname;
	my $filename;
	my $template;

	$name = shift;
	$subname = shift;

	if( defined $subname )
	{
		$filename = TEMPLATE_DIR . "/" . $subname . "/" . $name . ".tmpl" ;
	}
	else
	{
		$filename = TEMPLATE_DIR . "/" . $name . ".tmpl" ;
	}
	return undef if( ! -f $filename );

	$template = new HTML::Template	( 
									filename => $filename ,
									die_on_bad_params => 0,
									);
}


sub output
{
	my $text;
	my $opts;
	my $template;
	my $user;
	my $tmp;
	my $tmptab;
	my $nscans;
	my $pscans;
	$text = shift;
	$opts = shift;
	$template = open_template( "main" );

	#If init == 0, the database is probably done. So, you will not search the user in the database.
	if( $main->{'init'} == 0 )
	{
		$user = undef;
	}
	else
	{
		$user = authuser( $main );
	}

	if( defined $text )
	{
		$template->param( TEXT => $text );
	}
	else
	{
		$tmp = open_template( 'homepage' );
		if( defined $user )
		{
			$tmp->param( 'LOGGED' => 1 );
			$nscans = Nmap::Scan::getall( $main , { MAKESTR => 1 ,  LIMIT => 5 , ORDERBY => 'id' , INVERT => 1 , OWNER => $user->get( 'id' ) } );
			if( $#{@$nscans} > -1 )
			{
				$tmp->param( HAVE_SCANS => 1 );
				$tmp->param( NORMAL_SCANS => $nscans );
			}
		}
		$template->param( TEXT => $tmp->output );

	}

	if( defined $opts )
	{
		$template->param( WARNING => $opts->{WARN} ) if( defined $opts->{WARN} );
	}

	if( ( defined $user ) && ( defined( $user->get_theme ) ) )
	{
		$template->param( STYLE => $user->get_theme );
	}
	else
	{
		$template->param( STYLE => $main->get_config->get_default_theme  );
	}

	if( defined $user )
	{
		$template->param( LOGGED => 1 );
		$template->param( USERNAME => $user->get( 'username' ) );
		$template->param( ADMIN => 1 ) if( $user->get( 'state' ) == Nmap::User::STATE_ADMIN );
		foreach ( @{$main->get_config->get_themes} )
		{
			push( @$tmptab , { THEMENAME => $_ } );
		}
		$template->param( themes => $tmptab );
	}

	if( defined $opts->{REFRESH} )
	{
		$template->param( REFRESH => $opts->{REFRESH} );
		$template->param( REFRESH_TIME => $opts->{REFRESH_TIME} );
		$template->param( REFRESH_URL => $opts->{REFRESH_URL} );
	}

	if( defined $opts->{COOKIE} )
	{
		print CGI->header( -cookie => $opts->{COOKIE} , -type => "text/html" , -charset => "UTF-8" );
	}
	else
	{
		print CGI->header( -type => "text/html" , -charset => "UTF-8" );
	}
	print $template->output;
	
	exit( 0 );
}

sub check_param
{
	my $param = shift;
	my $opts  = shift;

	if( defined( $opts->{ALPHANUM} ) )
	{
		return( 1 ) if( ! ( $param =~ /^[a-zA-Z0-9]+$/ ) );
	}

	if( defined( $opts->{NUMONLY} ) )
	{
		return( 1 ) if( ! ( $param =~ /^[0-9]+$/ ) );
	}

	if( defined( $opts->{'MIN-LENGTH'} ) )
	{
		return( 1 ) if( length( $param ) < $opts->{'MIN-LENGTH'} );
	}

	if( defined( $opts->{'MAX-LENGTH'} ) )
	{
		return( 1 ) if( length( $param ) >= $opts->{'MAX-LENGTH'} );
	}



	if( defined( $opts->{NOMAJ} ) )
	{
		return( 1 ) if( $param =~ /[A-Z]/ );
	}

	if( defined( $opts->{NOMAJ} ) )
	{
		return( 1 ) if( $param =~ /[A-Z]/ );
	}

	if( defined( $opts->{EMAIL} ) )
	{
		return( 1 ) if( ! Email::Valid->address( $param ) );
	}

	if( defined( $opts->{HOST} ) )
	{
		return( 1 ) if( ! ($param =~ /^[a-zA-Z0-9\-\.]+$/ ) ) ;
	}

	return( 0 );
}

1;
__END__

=head1 SYNOPSIS

Nmap::CGI::Main some functions handle templates and print them

=head1 METHODS

=head2 init_web( {} )

Function to init a script for the web-app. In fact, it checks if a user is logged
if it's an admin, ... And can print errors according to the type of pages.
It returns an array which contains a reference to an Nmap::Main object, 
a reference to a CGI objet and a reference to a Nmap::User object if a user
is logged.

The argument is a hashtable which contains options. Options are hashkey. There is the
list of the possibles hashkey :

	* MUST-AUTH - The user logged must be auth

	* MUST-ADMIN - The logged user must be an admin

When MUST-AUTH or MUST-ADMIN are specified and if these conditions cannot be satisfied, then
a page with an error is returned to the client.

=head2 open_template( $tmplname , $directoryname )

Open a template and returns the correct HTML::Template reference.
If the second argument is omitted, will open the template called
$tmplname. If the second argument is present, will try to open 
template in the file $directoryname/$tmplname

=head2 output( $text , {} )

Will make a full output to send to the web-browser of the client. The first parameter
is the text to print. The second argument is a hashtable which contains some options.

In fact, this function will open the homepage template and put the first argument as
content of this template. 

The second argument is a hashtable. There is the list of the possible hashkeys :

	* REFRESH - The page must be refreshed

	* REFRESH_TIME - The page must be refreshed in hashvalue seconds

	* REFRESH_URL - When refreshed, the page will go to the URL hashvalue

	* WARNING - Print a warning to the page


=head2 check_param( $param , {} )

Check the param given in the first parameter. The case of checks is defined
by the hashkey in the second argument. You can specify several hashkey. If
one conditions is not satisfied, the functions returns 1. If success, returns 0.
There is the list of the available hashkey :
	
	* ALPHANUM - Only alphanumeric string

	* MIN-LENGTH - The hashvalue is the min length of the string
	
	* MAX-LENGTH - The hashvalue is the max length of the string
	
	* NUMONLY - The string is only composed by numbers
	
	* NOMAJ - No majuscule are allowed in the string
	
	* EMAIL - The string is an email
	
	* HOST - The string is an host



=head1 SEE ALSO

Nmap::User

=head1 AUTHOR

Julien Delange <julien at gunnm dot org>

=head1 COPYRIGHT

Julien Delange <julien at gunnm dot org>

