#!/usr/bin/perl -w

#Copyright (c) Nmap project and its contributor
#All rights reserved.

#Redistribution and use in source and binary forms, with or without 
#modification, are permitted provided that the following conditions 
#are met:

# 1. Redistributions of source code must retain the above copyright 
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in 
#    the documentation and/or other materials provided with the 
#    distribution.
# 3. Neither the name of Nmap nor the names of its contributors 
#    may be used to endorse or promote products derived from this 
#    software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.

package Nmap::Logger;
require Exporter;
@ISA 	= qw( Exporter );
@EXPORT = qw( nmaplog nmaplogevent );

use strict;

use constant
{
	LOGFILE => "/tmp/nmap-cgi-log",
	LOG		=> 1,
};


sub nmaplog
{
	my $str;
	my $fname;
	$str = shift;
	$fname = ">>" . LOGFILE ;
	return 0 if( LOG != 1 );
	open( FIC , $fname ) or return 1;

	print FIC $str . "\n";

	close( FIC ) or return( 3 );

	return( 0 );
}

sub nmaplogevent
{
	my ( $evt , $str );
	$evt = shift;
	$str = shift;

	return( nmaplog( $evt . ": " . $str ) );
}
__END__

=head1 SYNOPSIS

Nmap::Logger logs events from nmap-cgi.

=head1 DESCRIPTION

This module is an object which handle the logs of nmap-cgi. It is mainly used for debugging.

=head1 METHODS

=head2 nmaplog( SCALAR )

Add a string in the end of the logfile. The string added is the first argument.

=head2 nmaplogevent( SCALAR , SCALAR )

Add a string in the logfile. The line added in the file is like : "event: message".
The event is the first argument, the second is the message.

=head1 EXAMPLE
	
use Nmap::Logger;

nmaplog( "add a message in the logfile" );

nmaplogevent( "myevent" , "my message" );

=head1 SEE ALSO

Nmap::Main

=head1 AUTHOR

Julien Delange <julien at gunnm dot org>

=head1 COPYRIGHT

Julien Delange <julien at gunnm dot org>

