#!/usr/bin/perl -w -I/usr/share/nmap-cgi/api/
#Copyright (c) Nmap project and its contributor
#All rights reserved.

#Redistribution and use in source and binary forms, with or without 
#modification, are permitted provided that the following conditions 
#are met:

# 1. Redistributions of source code must retain the above copyright 
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in 
#    the documentation and/or other materials provided with the 
#    distribution.
# 3. Neither the name of Nmap nor the names of its contributors 
#    may be used to endorse or promote products derived from this 
#    software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.


use Nmap::CGI::Main;
use Nmap::User;
use Nmap::Functions;
use Nmap::Mail;
use Nmap::CGI::Auth;
use Nmap::Logger;
use strict;

my @ACTIONS = qw(register login logout forgot);

#local vars
my $cgi;
my $nmap;
my $text;
my $temp;
my $action;

( $nmap , $cgi ) = init_web();

$action = $cgi->param( 'action' );


if( ! ( defined $action ) )
{
	output( undef );
}
elsif( $action eq 'register' )
{
	my ($fname , $lname , $mail , $uname , $user , $address , $pwd , $code );
	$uname = $cgi->param( 'username' );
	$lname = $cgi->param( 'lastname' );
	$fname = $cgi->param( 'firstname' );
	$mail = $cgi->param( 'email' );
	$address = $cgi->param( 'address' );

	if( $nmap->get_config->get_subopen == 0 )
	{
		output( undef , { WARN => "Subscribe is not allowed" } );
	}


	if( defined $uname )
	{
		#We proceed to register on the platform here
		output( open_template( 'register' )->output , { WARN=> "Bad parameter for username"} ) if( check_param( $uname , { ALPHANUM => 1 , NOMAJ => 1} ) );
		output( open_template( 'register' )->output , { WARN=> "Bad parameter for email" } ) if( check_param( $mail , { EMAIL => 1  , 'MAX-LENGTH' => 100}) );
		output( open_template( 'register' )->output , { WARN=> "Bad parameter for firstname"} ) if( check_param( $fname ) );
		output( open_template( 'register' )->output , { WARN=> "Bad parameter for lastname"} ) if( check_param( $lname ) );
		
		$user = new Nmap::User( $nmap , $uname );
		output( open_template( 'register' )->output , { WARN => "User already registered" } ) if( $user->getdb );
		$user->set( 'firstname' , $fname );
		$user->set( 'lastname' , $lname );
		$user->set( 'mail' , $mail );
		$user->set( 'address' , $address );

		$pwd = gen_pwd();
		$code = gen_confirm();

		$user->set( 'password' , $pwd );
		$user->set( 'chash' , $code );


		if( $user->create > 0 )
		{
			sendmail( { NAME => $fname , PASSWORD => $pwd , CONFIRM_CODE => $code , mailname => "confirm" , to=> $mail , subject => "your request to the nmap platform" , LOGIN => $uname , FROM => $nmap->get_config->get_mailfrom } );

			output( open_template( 'homepage' )->output , {WARN=>"User has been created. A mail will be send will instructions to finish the creation of your account"} );
		}
		else
		{
			output( open_template( 'homepage' )->output , { WARN=> "An error occurs while registering your new user. Please try again" } );
		}
	}
	
	output( open_template( 'register' )->output );
}
elsif( $action eq 'login' )
{
	my ( $uname , $pass , $user , $cookie );
	$uname = addslashes( $cgi->param( 'login' ) );
	$pass = $cgi->param( 'password' );

	$user = new Nmap::User( $nmap , $uname );
	if( $user->getdb != 1 )
	{
		nmaplog( "Login failed with username " . $uname );
		output( open_template( 'homepage' )->output , { WARN => "Wrong username" } );
	}
	if( $user->get( 'state' ) == 0 )
	{
		nmaplog( "Login failed with username " . $uname );
		output( open_template( 'homepage' )->output , { WARN => "Registration not complete" } );
	}


	if( $user->pwdmatch( $pass ) )
	{
		nmaplog( "Login successfull with username " . $uname );
		$cookie = regauth( $user );

		output( "<h1>Auth in progess</h1>" , { WARN => " Please wait some seconds ..." , COOKIE => $cookie , REFRESH=>1 , REFRESH_TIME => 1 , REFRESH_URL => 'index.pl' } );
	}
	else
	{
		output( open_template( 'homepage' )->output , { WARN => "Wrong password" } );
	}


	output( "login" );
}
elsif( $action eq 'forgot' )
{
	my ( $user , $mail , $pwd );
	$temp = $cgi->param("name");
	if( defined $temp )
	{
		$user = new Nmap::User( $nmap , addslashes( $temp ) );
		if( $user->getdb != 1 )
		{
			output( open_template( 'forgot' )->output , {WARN=>"Wrong username"} );
		}
		if( $user->get( 'state' ) == Nmap::User::STATE_NEW )
		{
			output( open_template( 'forgot' )->output , {WARN=>"User not activated"} );
		}
		$mail = $user->get( 'mail' );
		$pwd = gen_pwd();
		$user->set( 'password' , $pwd );
		$user->save;

		sendmail( { NAME => $user->get( 'firstname' ) , PASSWORD => $pwd , mailname => "forgot" , to=> $mail , subject => "New password on nmap platform" } );
		output( open_template( 'forgot' )->output , {WARN=>"A mail has been sent with your new password" } );
	}
	else
	{
		output( open_template( 'forgot' )->output );
	}

}
elsif( $action eq 'confirm' )
{
	my ( $user , $tmp , $data , $name );
	$name = addslashes( $cgi->param( 'name' ) );
	$data = addslashes( $cgi->param( 'data' ) );
	
	$user = new Nmap::User( $nmap , $name  );

	if( $user->getdb != 1 )
	{
		output( open_template( 'homepage' )->output , {WARN=>"User does not exists"} );
	}

	$tmp = $user->get( 'chash' );

	if( $tmp eq $data )
	{
		$user->set( 'state' , Nmap::User::STATE_CONFIRMED );
		$user->save;
		output( open_template( 'homepage' )->output , {WARN=>"User updated. You can log in."} );
	}
	else
	{
		output( open_template( 'homepage' )->output , {WARN=>"Code does not match, please try again."} );
	}

}
else
{
	output( undef );
}
