#!/usr/bin/perl -w -I/usr/share/nmap-cgi/api/

#Copyright (c) Nmap project and its contributor
#All rights reserved.

#Redistribution and use in source and binary forms, with or without 
#modification, are permitted provided that the following conditions 
#are met:

# 1. Redistributions of source code must retain the above copyright 
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in 
#    the documentation and/or other materials provided with the 
#    distribution.
# 3. Neither the name of Nmap nor the names of its contributors 
#    may be used to endorse or promote products derived from this 
#    software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.

use Nmap::CGI::Main;
use Nmap::User;
use Nmap::Group;
use Nmap::Functions;
use Nmap::Mail;
use Nmap::CGI::Auth;
use Nmap::Logger;
use Nmap::Scan;

use Socket;
use Date::Simple qw( today );
use Time::ParseDate;

use strict;

use constant
{
	PROGRAM => "/usr/share/nmap-cgi/nmap-xml-diff",
};

my @ACTIONS = qw( create );

#local vars
my $cgi;
my $nmap;
my $text;
my $action;
my $user;
my $name;
my $opts;
my $tmp;
my @tmp;
my $tmpl;
my $type;
my $group;
my $sort_type;
my $invert;

( $nmap , $cgi , $user ) = init_web( { 'MUST-AUTH' => 1 } );

$action 		= $cgi->param( 'action' );
$type 			= $cgi->param( 'type' );
$name 			= $cgi->param( 'name' );
$sort_type 		= $cgi->param( "sort" );
$invert 		= $cgi->param( "invert" );


if( ! ( defined $action ) )
{
	output( open_template( 'homepage' )->output  );
}
elsif( $action eq 'create' )
{
	my $scan;
	my $sid;
	my $pass;
	my $type;
	my $pman;		#If it's a periodic scan and time is specified manually, it's stored in this var
	my $ptype;		#If the scan is a periodic scan, ptype is the type (weekly, daily, ...)
	my $ptime;
	my $stime;		#If it's a scheduled scan, it's the data when the scan should be processed
	my $tmp2;
	my $date;
	my ( $day , $month , $year , $hour , $min );


	#Init group, name and so on
	$name	= $cgi->param( 'hostname' );
	$type	= $cgi->param( 'SCANTYPE' );
	$ptype	= $cgi->param( 'ptype' );
	$pman	= $cgi->param( 'pman' );
	$stime	= $cgi->param( 'SCHEDULEDTIME' );
	$tmpl	= open_template( 'create' , 'scan' );
	$group	= $user->get_group();


	#Print the form to record a new scan
	#Prepare this form first, because if you have an error, we print it anyway
	$tmp = Nmap::Group::RIGHTS;
	foreach ( @$tmp )
	{
		if( $group->opt( $_ ) == 1 )
		{
			$tmpl->param( $_  => 1 );
		}
	}
	

	# save the current scan
	# redirect to a full edition with options
	# The user try to register a new scan
	if( defined $name )
	{
		if( ! defined( $type ) )
		{
			output( $tmpl->output , { WARN => 'Cannot find type' } );
		}
		
		
		if( $type == Nmap::Scan::TYPE_SINGLE )
		{
			output( $tmpl->output , { WARN => "You are not allowed to perform this sort of scan" } ) if( $group->opt( 'ALLOW-SCAN-SINGLE' ) != 1 );

			#Here, it's scan single
		}
		elsif( $type == Nmap::Scan::TYPE_SCHEDULED )
		{
			output( $tmpl->output , { WARN => "You are not allowed to perform this sort of scan" } ) if( $group->opt( 'ALLOW-SCAN-SCHEDULED' ) != 1 );

			output( $tmpl->output , { WARN => 'The time is not specified' } )if( ! defined( $stime ) );

			#Check the date
			if( ! ( ( $month , $day , $year , $hour , $min ) = ( $stime =~ /^(\d{2})\.(\d{2})\.(\d{4})\s(\d{2}):(\d{2})$/ ) ) )
			{
				output( $tmpl->output , { WARN => "Bad date format" } );
			}
			if	( 
					( ( $hour < 0 ) || ( $hour > 23 ) ) ||
					( ( $min < 0 ) || ( $min > 59 ) ) ||
					( ( $month < 1 ) || ( $month > 12 ) ) ||
					( ( $day < 1 ) || ( $day > 31 ) ) 
				)
			{
				output( $tmpl->output , { WARN => "Bad date, invalid input" } );
			}

			output( $tmpl->output , { WARN => "Bad year, invalid year" } ) if( $year < today()->year );
			if( $year == today()->year )
			{
				output( $tmpl->output , { WARN => "Bad date, invalid month" } ) if( $month < today()->month );
				
				if( $month == today()->month )
				{
					if( $day < today()->day )
					{
						output( $tmpl->output , { WARN => 'Bad date, invalid day' } );
					}
				}
			}

		}
		elsif( $type == Nmap::Scan::TYPE_PERIODIC )
		{
			output( $tmpl->output , { WARN => "You are not allowed to perform this sort of scan" } ) if( $group->opt( 'ALLOW-SCAN-PERIODIC' ) != 1 );
			if( $ptype == 3 )
			{
				$ptime = 604800;
			}
			elsif( $ptype == 2 )
			{
				$ptime = 86400;
			}
			elsif( $ptype == 1 )
			{
				$ptime = 3600;
			}
			elsif( $ptype == 0 )
			{
				output( $tmpl->output , { WARN => 'Unknow time' } ) if( ! defined( $pman ) );
				output( $tmpl->output , { WARN => 'Invalid time' } ) if( check_param( $pman , { 'NUMONLY' => 1 } ) != 0 );
				output( $tmpl->output , { WARN => 'Time must be more than 10 minutes' } ) if( $pman < 10 );
				$ptime = 60 * $pman;
			}
			else
			{
				output( $tmpl->output , { WARN => 'Unknow periodic type' } );
			}
			output( $tmpl->output , { WARN => 'The type must be specified' } )if( ! defined( $ptype ) );
		}
		else
		{
			output( $tmpl->output , { WARN => 'Invalid scan type' } );
		}

		#Some tests if the scan seems to be valid
		output( $tmpl->output , { WARN => 'Wrong target' } ) if( ! defined $name );
		output( $tmpl->output , { WARN => 'Erroneous target' } ) if( check_param( $name , { 'MAX-LENGTH' => 100 , 'MIN-LENGTH' => 1 } ) != 0  );

		#Create the main object
		$scan = new Nmap::Scan( $nmap , $name , $user , { 'TYPE' => $type } );
	
		output( $tmpl->output , { WARN => 'Bad paramaters' } ) if( ! defined( $scan ) );

		#The scan is an IP-address
		if( $scan->is_ip == 1 )
		{
			nmaplog( "the scan is an ip");
			my $t; #temporary var to resolv the hostname
			$t = gethostbyaddr( inet_aton( $scan->get( 'target' ) ) , AF_INET );

			#If group is only allowed to scan some domains
			if( $group->opt( 'ONLY-DOMAINS' ) == 1 )
			{
				$pass = 0;
				my @d = split( /\s*,\s*/ , $group->opt_value( 'ONLY-DOMAINS' ) );
				foreach ( @d )
				{
					if( $t =~ /$_$/ )
					{
						$pass = 1;
					}
				}
				if( $pass == 0 )
				{
					output( $tmpl->output , { WARN => 'You are not allowed to scan this target' } );
				}
			}


			if( ( defined $t ) && ( $group->opt( 'DENY-DOMAINS' ) == 1 ) )
			{
				my @d = split( /\s*,\s*/ , $group->opt_value( 'DENY-DOMAINS' ) );

				foreach ( @d )
				{
					if( $t =~ /$_$/ )
					{
						output( $tmpl->output , { WARN => 'You are not allowed to scan this target' } );
					}
				}
				
			}
		}
		elsif( $scan->is_subnet == 1 )
		{
			nmaplog( "the scan is a subnet");
			if( $group->opt( 'ALLOW-SUBNET' ) != 1 )
			{
				output( $tmpl->output , { WARN => 'You are not allowed to perform this type of scan' } );
			}
		}
		elsif( $scan->is_hostname == 1 )
		{
			nmaplog( "the scan is a hostname");
			$opts->{WARN} = "hostname";

			#If some domains are not allowed
			if( $group->opt( 'DENY-DOMAINS' ) == 1 )
			{
				my @d = split( /\s*,\s*/ , $group->opt_value( 'DENY-DOMAINS' ) );

				foreach ( @d )
				{
					if( $scan->get( 'target' ) =~ /$_$/ )
					{
						output( $tmpl->output , { WARN => 'You are not allowed to scan this target' } );
					}
				}
			}

			#If group is only allowed to scan some domains
			if( $group->opt( 'ONLY-DOMAINS' ) == 1 )
			{
				$pass = 0;
				my @d = split( /\s*,\s*/ , $group->opt_value( 'ONLY-DOMAINS' ) );
				foreach ( @d )
				{
					if( $scan->get( 'target' ) =~ /$_$/ )
					{
						$pass = 1;
					}
				}
				if( $pass == 0 )
				{
					output( $tmpl->output , { WARN => 'You are not allowed to scan this target' } );
				}
			}
		}
		else
		{
			output( $tmpl->output , { WARN => 'Invalid target' } );
		}

		#Check if the group of the user have quota and if the
		#number of scans repesct the quota
		if( ( $group->opt( 'MAX-SCANS-PER-DAY' ) == 1 ) &&
			( ( $type == Nmap::Scan::TYPE_SINGLE ) ||
			  ( $type == Nmap::Scan::TYPE_SCHEDULED ) ) )
		{
			$tmp2 = $group->opt_value( 'MAX-SCANS-PER-DAY' );
			$day = today()->day;
			$month = today()->month;
			$year = today()->year;
			$date = $year."-".$month."-".$day;
			$tmp = Nmap::Scan::getall( $nmap , { OWNER => $user->get( "id" ) , MINCREATEDATE => $date , 'SINGLE-SCHEDULED' => 1 } );
			if( $#{@$tmp} >= ( $tmp2 - 1 ) )
			{
				output( $tmpl->output , { WARN => 'Scan quota exceeded' } );
			}
		}

		if( ( $group->opt( 'MAX-SCANS-PER-WEEK' ) == 1 ) &&
			( ( $type == Nmap::Scan::TYPE_SINGLE ) ||
			  ( $type == Nmap::Scan::TYPE_SCHEDULED ) ) )
		{
			$tmp2 = $group->opt_value( 'MAX-SCANS-PER-WEEK' );
			$date = today() - 7;
			$tmp = Nmap::Scan::getall( $nmap , { OWNER => $user->get( "id" ) , MINCREATEDATE => $date , 'SINGLE-SCHEDULED' => 1 } );
			if( $#{@$tmp} >= ( $tmp2 - 1 ) )
			{
				output( $tmpl->output , { WARN => 'Scan quota exceeded' } );
			}
		}

		#Check ports range if the user specify one port range
		if( $group->opt( 'PORTS-RANGE' ) == 1 )
		{
			$tmp = $cgi->param( 'PORTS-RANGE' );
			$tmp2 = $cgi->param( 'PORTS-RANGE-VALUE' );
			if( ( $tmp == 1 ) && ( ! ( $tmp2 =~ /^\d[\d\-,]+\d$/ ) ) )
			{
				output( $tmpl->output , { WARN => "Bad port range" } );
			}
		}

		# Create the scan in the database
		# Will check for error after creating the object
		$sid = $scan->create;


		if( $sid < 1 )
		{
			$opts = { WARN => "Cannot create the scan" };
		}
		else
		{
			# Well, here, the scan is stored in the database
			# We have to save options in the scan
			# So, we fetch the scan from the database, and finally save options
			$scan = new Nmap::Scan( $nmap , $sid , $user , { METHOD => 'byid' } );

			#Get informations about the scan in the database
			$scan->getdb;
	
			#If the user is allowed to perform TCP scans, we find the tcp scan-type
			if( $group->opt( 'SCAN-TCP' ) == 1 )
			{
				$tmp = $cgi->param( 'SCAN-TCP' );
				$scan->setopt( 'SCAN-TCP' , $tmp ) if( ( defined $tmp ) && ( $tmp == 1 ) );

				$tmp = $cgi->param( 'TCPMETHOD' );
				$scan->setopt( 'TCP-METHOD' , $tmp ) if( ( defined $tmp ) && ( $tmp ne '' ) );
				$scan->setopt( 'SCAN-TCP' , 1 ) if( ( defined $tmp ) && ( $tmp ne '' ) && ( $tmp ne 'NOT' ) );
			}

			if( $group->opt( 'SCAN-UDP' ) == 1 )
			{
				$tmp = $cgi->param( 'SCAN-UDP' );
				$scan->setopt( 'SCAN-UDP' , $tmp ) if( ( defined $tmp ) && ( $tmp == 1 ) );
			}

			if( $group->opt( 'PING-TARGET' ) == 1 )
			{
				$tmp = $cgi->param( 'PING-TARGET' );
				$scan->setopt( 'PING-TARGET' , $tmp ) if( ( defined $tmp ) && ( $tmp == 1 ) );
			}

			if( $group->opt( 'PARALLELISM-CHANGE' ) == 1 )
			{
				$tmp = $cgi->param( 'MIN-PARALLELISM' );
				$scan->setopt( 'MIN-PARALLELISM' , $tmp ) if( ( defined $tmp ) && ( $tmp > 1 ) );

				$tmp = $cgi->param( 'MAX-PARALLELISM' );
				$scan->setopt( 'MAX-PARALLELISM' , $tmp ) if( ( defined $tmp ) && ( $tmp > 1 ) );
			}

			if( $group->opt( 'RTT-CHANGE' ) == 1 )
			{
				$tmp = $cgi->param( 'MIN-RTT' );
				$scan->setopt( 'MIN-RTT' , $tmp ) if( ( defined $tmp ) && ( $tmp > 1 ) );

				$tmp = $cgi->param( 'MAX-RTT' );
				$scan->setopt( 'MAX-RTT' , $tmp ) if( ( defined $tmp ) && ( $tmp > 1 ) );

				$tmp = $cgi->param( 'INITIAL-RTT' );
				$scan->setopt( 'INITIAL-RTT' , $tmp ) if( ( defined $tmp ) && ( $tmp > 1 ) );
			}

			if( $group->opt( 'OSDETECT' ) == 1 )
			{
				$tmp = $cgi->param( 'OSDETECT' );
				$scan->setopt( 'OSDETECT' , $tmp ) if( ( defined $tmp ) && ( $tmp == 1 ) );
				if( $group->opt( 'OSDETECT-AGRESSIVE' ) == 1 )
				{
					$tmp = $cgi->param( 'OSDETECT-AGRESSIVE' );
					$scan->setopt( 'OSDETECT-AGRESSIVE',$tmp ) if( ( defined $tmp ) && ( $tmp == 1 ) );
				}
			}

			if( $group->opt( 'ALLOW-IPV6' ) == 1 )
			{
				$tmp = $cgi->param( 'ALLOW-IPV6' );
				$scan->setopt( 'ALLOW-IPV6' , $tmp ) if( ( defined $tmp ) && ( $tmp == 1 ) );
			}

			if( $group->opt( 'VERSION' ) == 1 )
			{
				$tmp = $cgi->param( 'VERSION' );
				$scan->setopt( 'VERSION' , $tmp ) if( ( defined $tmp ) && ( $tmp == 1 ) );
			}

			if( $group->opt( 'MAIL-RESULT' ) == 1 )
			{
				$tmp = $cgi->param( 'MAIL-RESULT' );
				$scan->setopt( 'MAIL-RESULT' , $tmp ) if( ( defined $tmp ) && ( $tmp == 1 ) );
			}

			if( $group->opt( 'TREAT-ONLINE' ) == 1 )
			{
				$tmp = $cgi->param( 'TREAT-ONLINE' );
				$scan->setopt( 'TREAT-ONLINE' , $tmp ) if( ( defined $tmp ) && ( $tmp == 1 ) );
			}

			#We apply ports range
			#BEFORE creating the scan we check the argument with a regexp
			if( $group->opt( 'PORTS-RANGE' ) == 1 )
			{
				$tmp = $cgi->param( 'PORTS-RANGE' );
				$tmp2 = $cgi->param( 'PORTS-RANGE-VALUE' );
				$scan->setopt( 'PORTS-RANGE' , $tmp2 ) if( ( defined $tmp ) && ( $tmp == 1 ) );
			}

			if( $group->opt( 'TIMING-TEMPLATES' ) == 1 )
			{
				$tmp = $cgi->param( 'TIMING' );
				$tmp2 = $cgi->param( 'TIMING-VALUE' );
				#The value of the timing template is between 1 and 5 (included)
				if( ( defined( $tmp ) ) && ( $tmp == 1 ) && ( $tmp2 >= 0 ) && ( $tmp2 < 6 ) )
				{
					$scan->setopt( 'TIMING-TEMPLATE' , $tmp2 );
				}
			}


			#Stuff specific to SCHEDULED and PERIODIC type
			if( $type == Nmap::Scan::TYPE_SCHEDULED )
			{
				$tmp = parsedate( $stime );
				$scan->setopt( 'SCHEDULED-DATE' , $tmp ) if( defined( $tmp ) );
				$opts = { WARN => sprintf( "Scan is now scheduled" ) };
			}
			elsif( $type == Nmap::Scan::TYPE_PERIODIC )
			{
				$scan->setopt( 'PERIODIC-INTERVAL' , $ptime ) if( defined( $ptime ) );
				$opts = { WARN => sprintf( "Scan saved" ) };
			}
			elsif( $type == Nmap::Scan::TYPE_SINGLE )
			{
				#Get the number of scans waiting for processing
				#We think that a scan need 2 minutes to perform
				$tmp = Nmap::Scan::getall( $nmap ,  { STATE => Nmap::Scan::STATE_NEW , TYPE => Nmap::Scan::TYPE_SINGLE } );
				$tmp = $#{@$tmp} + 1;
				$opts = { WARN => sprintf( "Scan will be processed in %d minutes " , $tmp * 2 ) };
			}
			
			#Save the current scan in the database
			$scan->save;
		}
	}

	output( $tmpl->output , $opts );
}
elsif( $action eq 'list' )
{
	my $scans;

	if( $type eq 'single' )
	{
		$tmpl = open_template( 'list-single' , 'scan' );
		$scans = Nmap::Scan::getall( $nmap , { OWNER => $user->get('id') , MAKESTR => 1 , 'SINGLE-SCHEDULED' => 1 , NOSTATE => Nmap::Scan::STATE_FINISHED } );

		if( $#{@$scans} >= 0 )
		{
			if( $user->is_admin == 1 )
			{
				foreach ( @$scans )
				{
					$_->{admin} = 1;
				}
			}
			$tmpl->param( HAVE_SCANS => 1 );
			$tmpl->param( SCANS => $scans );
		}
		output( $tmpl->output  );
	}
	elsif( $type eq 'completed' )
	{
		$tmpl = open_template( 'list-completed' , 'scan' );

		$invert = 1 if( ! defined $invert );
		$sort_type="create_date" if( ! defined $sort_type );
		$scans = Nmap::Scan::getall( $nmap , { OWNER => $user->get('id') , STATE => Nmap::Scan::STATE_FINISHED , MAKESTR => 1  , 'SINGLE-SCHEDULED' => 1 , ORDERBY => $sort_type , INVERT => $invert } );

		if( $#{@$scans} >= 0 )
		{
			$tmpl->param( HAVE_SCANS => 1 );
			$tmpl->param( SCANS => $scans );
		}
		output( $tmpl->output  );
	}
	elsif( $type eq 'periodic' )
	{
		$tmpl = open_template( 'list-periodic' , 'scan' );
		$scans = Nmap::Scan::getall( $nmap , { OWNER => $user->get('id') , 'PERIODIC-PERIODICSTOPPED' => 1  , MAKESTR => 1 } );

		if( $#{@$scans} >= 0 )
		{
			$tmpl->param( HAVE_SCANS => 1 );
			$tmpl->param( SCANS => $scans );
		}
		output( $tmpl->output  );
	}
}
elsif( $action eq 'view' )
{
	my $scan;
	my $sons;
	output( undef  , { WARN => "Unknow id" } )if( ! defined $name );
	$scan = new Nmap::Scan( $nmap , $name , undef , { METHOD => 'byid' } );
	output( undef  , { WARN => "Unknow scan object" } )if( ! defined $scan );
	output( undef  , { WARN => "Unable to get object" } )if( $scan->getdb != 1 );

	if( ( $scan->get( 'uidowner' ) != $user->get_id ) && ($user->is_admin != 1 ) )
	{
		output( undef , { WARN => "You are not allowed to see that" } );
	}

	if( ( $scan->get( 'type' ) == Nmap::Scan::TYPE_PERIODIC ) || ( $scan->get( 'type' ) == Nmap::Scan::TYPE_PERIODICSTOPPED ) )
	{
		$sons = Nmap::Scan::getall( $nmap , { FATHER=>$scan->get('id') , OWNER => $scan->get( 'uidowner' ) , ORDERBY => 'id' , INVERT => '1'  , STATE => Nmap::Scan::STATE_FINISHED });
		$tmpl = open_template( 'view-periodic' , 'scan' );
		$scan->merge_opts;
		$scan->make_str;

		foreach ( @$sons )
		{
			$_->make_str;
		}

		if( $#{@$sons} >= 0 )
		{
			#We add the epoch2date attribute to the scan
			$scan->{HAVE_INHERITED} = 1;
			$scan->{scans} = $sons;
		}
		$tmpl->param( SCAN => [ $scan ] );
		output( $tmpl->output  );
	}
	else
	{
		$tmpl = open_template( 'view-single' , 'scan' );
		$scan->merge_opts;
		$scan->make_str;
		$scan->{'finished'} = 1 if( $scan->get( 'state' ) == Nmap::Scan::STATE_FINISHED );
		$tmpl->param( SCAN => [ $scan ] );
		output( $tmpl->output  );
	}
}
elsif( $action eq 'convert' )
{
	if( $type eq 'xml' )
	{
		my $scan;
		my $filename;
		output( undef  , { WARN => "Unknow id" } )if( ! defined $name );
		$scan = new Nmap::Scan( $nmap , $name , undef , { METHOD => 'byid' } );
		output( undef  , { WARN => "Unknow scan object" } )if( ! defined $scan );
		output( undef  , { WARN => "Unable to get object" } )if( $scan->getdb != 1 );

		if( ( $scan->get( 'uidowner' ) != $user->get( 'id' ) ) &&
			( $user->is_admin != 1 ) )
			{
				output( undef , { WARN => "You are not allowed to see it" } );
			}
			

		$filename = $scan->get_xmlfile;
		if( ! -f $filename )
		{
			output( undef , { WARN => "Error when get the XML relevant content" } );
		}
		print CGI->header( -type => "text/xml" );
		open FILE , $filename;
		while( <FILE> )
		{
			if( /nmap\.xsl/)
			{
				print "<?xml-stylesheet href=\"/themes/".$user->get_theme."/nmap.xsl\" type=\"text/xsl\"?>\n";
			}
			else
			{
				print $_;
			}
		}
		close FILE;
		exit( 0 );
		
	}
	if( $type eq 'txt' )
	{
		my $scan;
		my $filename;
		output( undef  , { WARN => "Unknow id" } )if( ! defined $name );
		$scan = new Nmap::Scan( $nmap , $name , undef , { METHOD => 'byid' } );
		output( undef  , { WARN => "Unknow scan object" } )if( ! defined $scan );
		output( undef  , { WARN => "Unable to get object" } )if( $scan->getdb != 1 );

		if( ( $scan->get( 'uidowner' ) != $user->get( 'id' ) ) &&
			( $user->is_admin != 1 ) )
			{
				output( undef , { WARN => "You are not allowed to see it" } );
			}
			

		$filename = $scan->get_txtfile;
		if( ! -f $filename )
		{
			output( undef , { WARN => "Sorry, the file is not available" } );
		}
		print CGI->header( -type => "text/plain" );
		open FILE , $filename;
		while( <FILE> )
		{
			print $_;
		}
		close FILE;
		exit( 0 );
	}
		
}
elsif( $action eq 'delete' )
{
	my $scan;
	output( undef  , { WARN => "Unknow id" } )if( ! defined $name );
	$scan = new Nmap::Scan( $nmap , $name , undef , { METHOD => 'byid' } );
	output( undef  , { WARN => "Unknow scan object" } )if( ! defined $scan );
	output( undef  , { WARN => "Unable to get object" } )if( $scan->getdb != 1 );
	$tmp = CGI::referer();

	if( ( $scan->get( 'uidowner' ) != $user->get( 'id' ) ) &&
		( $user->is_admin != 1 ) )
		{
			output( undef , { WARN => "You are not allowed to see it" } );
		}
	if( $scan->delete  == 1 )
	{
		output( undef , { WARN => "Scan successfully deleted" , REFRESH => 2 , REFRESH_URL => $tmp , REFRESH_TIME => 0  } );
	}
	else
	{
		output( undef , { WARN => "Cannot delete scan" } );
	}
}
elsif( $action eq 'compare' )
{
	my( $oriid , $newid );
	my( $oriscan , $newscan );
	my( $orifile , $newfile );
	$oriid = $cgi->param( 'ori' );
	$newid = $cgi->param( 'new' );

	output( undef , { WARN => "Error on id" } ) if( ! defined( $oriid ) );
	output( undef , { WARN => "Error on id" } ) if( ! defined( $newid ) );

	$oriscan = new Nmap::Scan( $nmap , $oriid , undef , { METHOD => 'byid' } );
	$newscan = new Nmap::Scan( $nmap , $newid , undef , { METHOD => 'byid' } );
	output( undef , { WARN => "Error on scan" } ) if( ! defined( $oriscan ) );
	output( undef , { WARN => "Error on scan" } ) if( ! defined( $newscan ) );
	output( undef , { WARN => "Cannot fetch scan" } ) if( $oriscan->getdb != 1 );
	output( undef , { WARN => "Cannot fetch scan" } ) if( $newscan->getdb != 1 );

	output( undef , { WARN => "Original scan is not scheduled or single" } ) if( ( $oriscan->get( 'type')  != Nmap::Scan::TYPE_SINGLE ) && ( $oriscan->get( 'type' )  != Nmap::Scan::TYPE_SCHEDULED ) );
	output( undef , { WARN => "New scan is not scheduled or single" } ) if( ( $newscan->get( 'type' )  != Nmap::Scan::TYPE_SINGLE ) && ( $newscan->get( 'type' )  != Nmap::Scan::TYPE_SCHEDULED ) );


	#Check if the user that ask the comparaison
	#is the owner of the scans or an admin
	if( 
		( $oriscan->get( "uidowner" ) != $user->get( "id" ) )&&
		( $user->is_admin != 1 )
		)
		{
			output( undef , { WARN => "You're not allowed to see that" } );
		}
	if( 
		( $newscan->get( "uidowner" ) != $user->get( "id" ) )&&
		( $user->is_admin != 1 )
		)
		{
			output( undef , { WARN => "You're not allowed to see that" } );
		}
	$oriscan->make_str;
	$newscan->make_str;
	$oriscan->merge_opts;
	$newscan->merge_opts;
	$tmpl = open_template( 'compare' , 'scan' );
	$tmpl->param( ORISCAN => [ $oriscan ] );
	$tmpl->param( NEWSCAN => [ $newscan ] );
	$tmpl->param( ORI_ID => $oriscan->get( "id" ) );
	$tmpl->param( NEW_ID => $newscan->get( "id" ) );

	output( $tmpl->output );

}
elsif( $action eq 'dlcompare' )
{
	my( $oriid , $newid );
	my( $oriscan , $newscan );
	my( $orifile , $newfile );
	$oriid = $cgi->param( 'ori' );
	$newid = $cgi->param( 'new' );

	output( undef , { WARN => "Error on id" } ) if( ! defined( $oriid ) );
	output( undef , { WARN => "Error on id" } ) if( ! defined( $newid ) );

	$oriscan = new Nmap::Scan( $nmap , $oriid , undef , { METHOD => 'byid' } );
	$newscan = new Nmap::Scan( $nmap , $newid , undef , { METHOD => 'byid' } );
	output( undef , { WARN => "Error on scan" } ) if( ! defined( $oriscan ) );
	output( undef , { WARN => "Error on scan" } ) if( ! defined( $newscan ) );
	output( undef , { WARN => "Cannot fetch scan" } ) if( $oriscan->getdb != 1 );
	output( undef , { WARN => "Cannot fetch scan" } ) if( $newscan->getdb != 1 );

	output( undef , { WARN => "Scans must be finished" } ) if( $oriscan->get( 'state' ) != Nmap::Scan::STATE_FINISHED );
	output( undef , { WARN => "Scans must be finished" } ) if( $newscan->get( 'state' ) != Nmap::Scan::STATE_FINISHED );


	#Check if the user that ask the comparaison
	#is the owner of the scans or an admin
	if( 
		( $oriscan->get( "uidowner" ) != $user->get( "id" ) )&&
		( $user->is_admin != 1 )
		)
		{
			output( undef , { WARN => "You're not allowed to see that" } );
		}
	if( 
		( $newscan->get( "uidowner" ) != $user->get( "id" ) )&&
		( $user->is_admin != 1 )
		)
		{
			output( undef , { WARN => "You're not allowed to see that" } );
		}


	$orifile = $oriscan->get_xmlfile;
	$newfile = $newscan->get_xmlfile;
	output( undef , { WARN => "Cannot find XML results" } ) if( ! -r $orifile );
	output( undef , { WARN => "Cannot find XML results" } ) if( ! -r $newfile );

	if( ! -x PROGRAM )
	{
		output( undef , { WARN => "Bad XMLdiff program" } ) if( ! -f $newfile );
	}

	open MYPIPE , PROGRAM . " $orifile $newfile |";

	print CGI->header( -type => "text/xml" );
	while( <MYPIPE> )
	{
		if( $_ =~ /nmap\-cgi\.xsl/ )
		{
			print "<?xml-stylesheet href=\"/themes/".$user->get_theme."/compare.xsl\" type=\"text/xsl\"?>\n";
		}
		else
		{
			print $_;
		}

	}
	close( MYPIPE );
	exit;
}
elsif( $action eq 'post' )
{
	my $scan;
	my ( $text , $title );

	$title = $cgi->param( 'title' );
	$text = $cgi->param( 'comment' );

	
	output( undef  , { WARN => "Bad title" } )if( ! defined $title );
	output( undef  , { WARN => "Bad comment" } )if( ! defined $text );
	output( undef  , { WARN => "Unknow id" } )if( ! defined $name );
	$scan = new Nmap::Scan( $nmap , $name , undef , { METHOD => 'byid' } );
	output( undef  , { WARN => "Unknow scan object" } )if( ! defined $scan );
	output( undef  , { WARN => "Unable to get object" } )if( $scan->getdb != 1 );
	$tmp = CGI::referer();

	if( ( $scan->get( 'uidowner' ) != $user->get( 'id' ) ) &&
		( $user->is_admin != 1 ) )
		{
			output( undef , { WARN => "You are not allowed to see it" } );
		}
	if( $scan->addpost( $title , $text , $user->get( 'id' ) )  == 1 )
	{
		output( undef , { WARN => "Successfully add this comment !" , REFRESH => 1 , REFRESH_URL => $tmp , REFRESH_TIME => 0  } );
	}
	else
	{
		output( undef , { WARN => "Post error" } );
	}
}
elsif( $action eq 'delpost' )
{
	my $scan;
	my $cid;

	$cid = $cgi->param( 'cid' );


	output( undef  , { WARN => "Bad command" } )if( ! defined $cid );
	output( undef  , { WARN => "Unknow id" } )if( ! defined $name );
	$scan = new Nmap::Scan( $nmap , $name , undef , { METHOD => 'byid' } );
	output( undef  , { WARN => "Unknow scan object" } )if( ! defined $scan );
	output( undef  , { WARN => "Unable to get object" } )if( $scan->getdb != 1 );
	$tmp = CGI::referer();

	if( ( $scan->get( 'uidowner' ) != $user->get( 'id' ) ) &&
		( $user->is_admin != 1 ) )
		{
			output( undef , { WARN => "You are not allowed to make that" } );
		}
	if( $scan->delpost( $cid )  == 1 )
	{
		output( undef , { WARN => "Post deleted" , REFRESH => 1 , REFRESH_URL => $tmp , REFRESH_TIME => 0  } );
	}
	else
	{
		output( undef , { WARN => "Error when delete port" } );
	}
}
elsif( $action eq 'active' )
{
	my $scan;
	output( undef  , { WARN => "Unknow id" } )if( ! defined $name );
	$scan = new Nmap::Scan( $nmap , $name , undef , { METHOD => 'byid' } );
	output( undef  , { WARN => "Unknow scan object" } )if( ! defined $scan );
	output( undef  , { WARN => "Unable to get object" } )if( $scan->getdb != 1 );
	$tmp = CGI::referer();

	if( ( $scan->get( 'uidowner' ) != $user->get( 'id' ) ) &&
		( $user->is_admin != 1 ) )
		{
			output( undef , { WARN => "You are not allowed to see it" } );
		}

	if	(
		( $scan->get( 'type' ) != Nmap::Scan::TYPE_PERIODIC ) &&
		( $scan->get( 'type' ) != Nmap::Scan::TYPE_PERIODICSTOPPED ) )
		{
			output( undef , { WARN => "Is not a periodic scan" } );
		}


	if( $scan->get('type') == Nmap::Scan::TYPE_PERIODIC )
	{
		$scan->set( 'type' , Nmap::Scan::TYPE_PERIODICSTOPPED );
	}
	else
	{
		$scan->set( 'type' , Nmap::Scan::TYPE_PERIODIC );
	}

	if( $scan->save  == 1 )
	{
		output( undef , { WARN => "Changed, will be redirected" , REFRESH => 1 , REFRESH_URL => $tmp , REFRESH_TIME => 0  } );
	}
	else
	{
		output( undef , { WARN => "Errors when change" } );
	}

}


output( undef , { WARN => "Unknow action and/or type" } );
