#!/usr/bin/perl -w -I/usr/share/nmap-cgi/api/
#Copyright (c) Nmap project and its contributor
#All rights reserved.

#Redistribution and use in source and binary forms, with or without 
#modification, are permitted provided that the following conditions 
#are met:

# 1. Redistributions of source code must retain the above copyright 
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in 
#    the documentation and/or other materials provided with the 
#    distribution.
# 3. Neither the name of Nmap nor the names of its contributors 
#    may be used to endorse or promote products derived from this 
#    software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.


use CGI;
use Nmap::CGI::Main;
use Nmap::User;
use Nmap::Functions;
use Nmap::Mail;
use Nmap::Scan;
use Nmap::CGI::Auth;
use strict;

my @ACTIONS = qw( edit );

#local vars
my $cgi;
my $nmap;
my $text;
my $action;
my $user;
my $name;
my $tmpl;
my $tmp;

( $nmap , $cgi , $user ) = init_web( { 'MUST-AUTH' => 1 } );

$action = $cgi->param( 'action' );


if( ! ( defined $action ) )
{
	output( open_template( 'homepage' )->output  );
}
elsif( $action eq 'logout' )
{
	delete_auth;
	output( open_template( 'homepage' )->output , { WARN => "You have been logged off" } );
}
elsif( $action eq 'edit' )
{
	my ( $tmp , $opts , $pass );
	$name = $cgi->param( 'firstname' );

	$tmpl = open_template( 'profile' , 'user' );
	

	if( defined $name )
	{
		$pass = 1;
		foreach ( qw( address  mail  firstname  lastname ) )
		{
			if( $_ eq 'mail' )
			{
				if( Nmap::CGI::Main::check_param( $cgi->param( $_ ) , { EMAIL => 1 } ) != 0 )
				{
					$pass = 0;
					$opts = { WARN => "Email is not valid" };
				}
			}
			$tmp = $cgi->param( $_ );
			$user->set( $_ , $tmp ) if( defined $tmp );
		}

		if( ( $pass == 1 ) && ( $user->save == 1 ) )
		{
			#If all was good, we save the user and print a message
			$opts = { WARN => "Changes saved" };
		}
		else
		{
			#If no error was declared in opts, we fill it
			$opts = { WARN => "Bad parameter, error when save" }if( ! defined( $opts ) );
		}
	}

	foreach( qw( username firstname lastname mail address ) )
	{
		$tmpl->param( $_ , $user->get( $_ ) );
	}

	output( $tmpl->output , $opts );
}
elsif( $action eq 'password' )
{
	my( $pass1 , $pass2 );
	$pass1 = $cgi->param( 'password1' );
	$pass2 = $cgi->param( 'password2' );


	$tmpl = open_template( 'profile' , 'user' );
	foreach( qw( username firstname lastname mail address ) )
	{
		$tmpl->param( $_ , $user->get( $_ ) );
	}

	if( $pass1 eq $pass2 )
	{

		if( check_param( $pass1 , { 'MIN-LENGTH' => 4 , 'MAX_LENGTH' => 12 } ) != 0 )
		{
			output( $tmpl->output , { WARN => "Invalid password. Must contains at least 4 caracters"} );
		}
		$user->set( 'password' , $pass1 );
		$user->save;
		output( $tmpl->output , { WARN => "Password changed" } );
	}

	output( $tmpl->output , { WARN => "Passwords does not match" } );

}
elsif( $action eq 'changetheme' )
{
	$name = $cgi->param( 'name' );
	$user->set( "theme" , $name );
	$user->save;
	$tmp = CGI::referer();
	#After registering the new theme, try to go on the page
	#before click on the new theme
	if( defined( $tmp ) )
	{
		output( undef , { WARN => "Theme changed" , REFRESH => 1 , REFRESH_URL => $tmp , REFRESH_TIME => 0 } );
	}
	else
	{
		output( undef , { WARN => "Theme changed" } );
	}
}
elsif( $action eq 'stats' )
{
	my $scans;
	my $s;
	my $tmp;
	my $t;
	my %hits;
	my @tab;

	$tmpl = open_template( 'stats' , 'user' );
	$scans = Nmap::Scan::getall( $nmap , { 'OWNER' => $user->get( 'id' ) , 'TYPE' => Nmap::Scan::TYPE_PERIODIC } );
	$tmpl->param( 'NB_SCAN_PERIODIC' =>  $#{@{$scans}} + 1 );

	$scans = Nmap::Scan::getall( $nmap , { 'OWNER' => $user->get( 'id' ) , 'NOTYPE' => Nmap::Scan::TYPE_PERIODIC } );

	$tmp = 0;
	foreach $s ( @{$scans} )
	{
		$tmp++;
		$hits{$s->get( 'target' )}++;
	}

	$tmpl->param( "NB_SCAN_NORMAL" =>  $tmp );

	$tmp = 0;
	$t = "";
	foreach $s ( keys %hits )
	{
		if( $hits{$s} > $tmp )
		{
			$tmp = $hits{$s};
			$t = $s;
		}
	}

	$tmpl->param( 'MOST_SCANNED' =>  $t );


	@tab = keys( %hits );
	$tmpl->param( "NB_DISTINCT_HOST" =>  $#tab + 1 );

	output( $tmpl->output  );
}
else
{
	output( open_template( 'homepage' )->output  );
}
output( undef , { WARN => "Unknow action/type" } );
