/*
Copyright (c) Nmap project and its contributors
All rights reserved.

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions 
are met:

 1. Redistributions of source code must retain the above copyright 
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in 
    the documentation and/or other materials provided with the 
    distribution.
 3. Neither the name of Nmap nor the names of its contributors 
    may be used to endorse or promote products derived from this 
    software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
 INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
 CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
 LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
 ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 POSSIBILITY OF SUCH DAMAGE.
*/

#include <string.h>
#include <stdlib.h>
#include <libpq-fe.h>

#include <config.h>

/* check the connection to the database
 * returns 0 if connection is OK
 * return 1 otherwise
 */
int check_conn( PGconn* conn )
{
	if( PQstatus( conn ) != CONNECTION_OK )
		return( 0 );
	return( 1 );
}

/* Connect to the database according to the parameters
 * stored in the config (given in parameter)
 * Return a database handle
 */
PGconn* db_connect( config_t* config )
{
	char params[1024];
	char *tmp;
	PGconn* co;

	memset( params , '\0' , 1024 );
	/* Check if the name of the database is present and
	 * not empty
	 */
	if( config->dbname != NULL )
	{
		tmp = config->dbname;
		if( strlen( tmp ) > 0 )
		{
			strcat( params , "dbname=" );
			strcat( params , tmp );
			strcat( params , " " );
		}
	}


	if( config->host != NULL )
	{
		tmp = config->host;
		if( strlen( tmp ) > 0 )
		{
			strcat( params , "host=" );
			strcat( params , tmp );
			strcat( params , " " );
		}
	}

	if( config->port != NULL )
	{
		tmp = config->port;
		if( strlen( tmp ) > 0 )
		{
			strcat( params , "port=" );
			strcat( params , tmp );
			strcat( params , " " );
		}
	}

	if( config->dbuser != NULL )
	{
		tmp = config->dbuser;
		if( strlen( tmp ) > 0 )
		{
			strcat( params , "user=" );
			strcat( params , tmp );
			strcat( params , " " );
		}
	}
	if( config->dbpass != NULL )
	{
		tmp = config->dbpass;
		if( strlen( tmp ) > 0 )
		{
			strcat( params , "password=" );
			strcat( params , tmp );
			strcat( params , " " );
		}
	}

	co = PQconnectdb( params );

	if( ! check_conn( co ) )
		return( NULL );
	return( co );
}

/* Close database connection */
void dbclose( PGconn *co )
{
	PQfinish( co );
}


int getdb( PGconn *co )
{
	PGresult	*res;

	res = PQexec( co , "BEGIN");

	if( PQresultStatus( res ) != PGRES_COMMAND_OK )
	{
		PQclear( res );
		return( 0 );
	}
	PQclear( res );
	return( 1 );
}

int releasedb( PGconn *co )
{
	PGresult	*res;

	res = PQexec( co , "END");

	if( PQresultStatus( res ) != PGRES_COMMAND_OK )
	{
		PQclear( res );
		return( 0 );
	}
	PQclear( res );
	return( 1 );
}

