/*
Copyright (c) nmap project and its contributors
All rights reserved.

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions 
are met:

 1. Redistributions of source code must retain the above copyright 
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in 
    the documentation and/or other materials provided with the 
    distribution.
 3. Neither the name of Nmap nor the names of its contributors 
    may be used to endorse or promote products derived from this 
    software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
 INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
 CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
 LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
 ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 POSSIBILITY OF SUCH DAMAGE.
*/

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>


#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>

#include <scan.h>
#include <config.h>
#include <request.h>
#include <debug.h>
#include <logger.h>
#include <mail.h>


extern config_t* config;
extern int nullfd;
int nbscans;
scan_t scans[MAXSCANS];

void init_scans()
{
	nbscans = 0;
}

int can_run( )
{
	if( nbscans < MAXSCANS )
		return( 1 );
	return( 0 );
}

int wait_scans()
{
	int i,j;
	int pid;
	int status;

	pid = waitpid( -1 , &status , WNOHANG );
	if( pid < 1 )
		return( 0 );
	for( i = 0 ; i < nbscans ; i++ )
	{
		if( pid == scans[i].pid )
		{
			if( DEBUG )
			{
				fprintf( stderr , "We found the process id %d\n" , pid );
			}
			/* Check return of the scan */
			if( WEXITSTATUS( status ) != 0 )
			{
				mark_scan( scans[i].scanid , STATE_EXECERROR );
				if( have_mailtag( scans[i].scanid ) == 1 )
					mail_error( scans[i].scanid );
			}
			else
			{
				mark_scan( scans[i].scanid ,STATE_FINISHED );
				if( have_mailtag( scans[i].scanid ) == 1 )
					mail_result( scans[i].scanid );
			}
			update_date( scans[i].scanid );

			for( j = i ; j < nbscans ; j++ )
			{
				scans[j].scanid = scans[j+1].scanid;
				scans[j].pid = scans[j+1].pid;
			}
			nbscans--;
			
			return( 1 );
		}
	}
	return( 0 );
}

int create_directory( const int scanid )
{
	int			d,h,t;
	struct stat	sstat;
	char	tmp[200];
	d = scanid/10;
	h = scanid/100;
	t = scanid/1000;



	sprintf( tmp , "%s/%d" , config->datadir , t );
	if( stat( tmp , &sstat ) == -1 )
	{
		if( mkdir( tmp , 0755 ) == -1 )
			return( -1 );
	}
	else
	{
		if( ! S_ISDIR( sstat.st_mode ) )
		{
			return( -1 );
		}
	}

	sprintf( tmp , "%s/%d/%d" , config->datadir , t , h );
	if( stat( tmp , &sstat ) == -1 )
	{
		if( mkdir( tmp , 0755 ) == -1 )
			return( -1 );
	}
	else
	{
		if( ! S_ISDIR( sstat.st_mode ) )
		{
			return( -1 );
		}
	}

	sprintf( tmp , "%s/%d/%d/%d" , config->datadir , t , h , d );
	if( stat( tmp , &sstat ) == -1 )
	{
		if( mkdir( tmp , 0755 ) == -1 )
			return( -1 );
	}
	else
	{
		if( ! S_ISDIR( sstat.st_mode ) )
		{
			return( -1 );
		}
	}

	return( 1);
}

char* find_xmlfilename( const int scanid )
{
	char*	tmp;
	int		d,h,t;
	struct stat	sstat;
	tmp = calloc( 200 , sizeof( char ) );

	d = scanid/10;
	h = scanid/100;
	t = scanid/1000;
	sprintf( tmp , "%s/%d/%d/%d" , config->datadir , t , h , d );

	if( stat( tmp , &sstat ) == -1 )
	{
		if( create_directory( scanid ) == -1 )
		{
			free( tmp );
			return( NULL );
		}
	}
	else
	{
		if( ! S_ISDIR( sstat.st_mode ) )
		{
			free( tmp );
			return( NULL );
		}
	}

	sprintf( tmp , "%s/%d/%d/%d/%d.xml" , config->datadir , t , h , d , scanid );
	return( tmp );
}

char* find_txtfilename( const int scanid )
{
	char*	tmp;
	int		d,h,t;
	struct stat	sstat;
	tmp = calloc( 200 , sizeof( char ) );

	d = scanid/10;
	h = scanid/100;
	t = scanid/1000;
	sprintf( tmp , "%s/%d/%d/%d" , config->datadir , t , h , d );

	if( stat( tmp , &sstat ) == -1 )
	{
		if( create_directory( scanid ) == -1 )
		{
			free( tmp );
			return( NULL );
		}
	}
	else
	{
		if( ! S_ISDIR( sstat.st_mode ) )
		{
			free( tmp );
			return( NULL );
		}
	}

	sprintf( tmp , "%s/%d/%d/%d/%d.txt" , config->datadir , t , h , d , scanid );
	return( tmp );
}


/*
 * See if scan should be performed and start them if needed
 */
int perform_scans( )
{
	int scanid;
	char **args;
	int pid;
	int i;
	char debug[1024];

	scanid = get_scanid( );

	if( nbscans >= MAXSCANS )
	{
		return( 0 );
	}

	if( scanid <= 0 )
	{
		return( 0 );
	}

	if( mark_scan( scanid , STATE_WIP ) != 1 )
	{
		DEBUGMSG( "Cannot mark done scan" );
		return( -1 );
	}
	
	args = scan2args( scanid );
	if( args == NULL )
	{
		DEBUGMSG( "Cannot find args" );
		return( -2 );
	}


	memset( debug , '\0' , 1024 );
	sprintf( debug , "Scan #%d: " , scanid );
	for( i = 0 ; i < MAXARGS ; i++ )
	{
		if( args[i] == NULL )
			break;
		strcat( debug , args[i] );
		strncat( debug , " " , 1 );
	}
	DEBUGMSG( debug );
	nmaplog( debug );

	pid = fork();

	switch( pid )
	{
		case 0:
			/* Redirect to /dev/null nmap output */
			dup2( nullfd ,STDOUT_FILENO );
			dup2( nullfd ,STDERR_FILENO );
			execv( config->cmd , args );
			exit( EXIT_SUCCESS ); 
			break;
		case -1:
			DEBUGMSG( "error when fork()" );
			break;
		default:
			scans[nbscans].pid = pid;
			scans[nbscans].scanid = scanid;
			nbscans++;
			break;
	}
	return( 1 );
}


